// $Id: AlienCommandModel.cpp,v 1.8 2012/10/16 18:51:27 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "AlienCommandModel.h"

#include "CustomPixmap.h"
#include "Icons.h"
#include "IconEngine.h"
#include "IconSize.h"
#include "TimeStamp.h"
#include "XmlOptions.h"

#include <QIcon>
#include <cassert>

//_______________________________________________
const QString AlienCommandModel::columnTitles_[ AlienCommandModel::nColumns ] =
{ "Id", "Priority", "Name", "Arguments" };

//_______________________________________________________________
AlienCommandModel::AlienCommandModel( QObject* parent ):
    ListModel<AlienCommand>( parent ),
    Counter( "AlienCommandModel" )
{

    Debug::Throw( "AlienCommandModel::AlienCommandModel.\n" );
    updateConfiguration();

}

//__________________________________________________________________
QVariant AlienCommandModel::data( const QModelIndex& index, int role ) const
{

    // check index, role and column
    if( !index.isValid() ) return QVariant();

    // retrieve associated command
    const AlienCommand& command( get()[index.row()] );

    // return text associated to command and column
    if( role == Qt::DisplayRole ) {

        switch( index.column() )
        {

            case ID:
            return command.id();

            case NAME:
            return command.name();

            case ARGUMENTS:
            return command.arguments().join( " " );

            case PRIORITY:
            return command.priority();

            default:
            return QVariant();
        }
    }

    // return icon associated to file
    if( role == Qt::DecorationRole && index.column() == NAME )
    { return _icon(command.type()); }

    return QVariant();

}

//__________________________________________________________________
QVariant AlienCommandModel::headerData(int section, Qt::Orientation orientation, int role) const
{

    if(
        orientation == Qt::Horizontal &&
        role == Qt::DisplayRole &&
        section >= 0 &&
        section < nColumns )
    { return columnTitles_[section]; }

    // return empty
    return QVariant();

}

//_________________________________________________________
void AlienCommandModel::updateConfiguration( void )
{

    Debug::Throw( "AlienCommandModel::updateConfiguration.\n" );

    // install pixmaps
    _icons().clear();
    _installIcons();

    reset();

}

//_____________________________________________________________________
void AlienCommandModel::_installIcons( void ) const
{

    Debug::Throw( "AlienCommandList::_installIcons" );

    if( !_icons().empty() ) return;

    // type icons
    typedef QHash< AlienCommand::Type, QString > IconNames;
    IconNames typeNames;
    typeNames[AlienCommand::MASTER_LIST] = ICONS::RELOAD;
    typeNames[AlienCommand::CHILDREN_LIST] = ICONS::RELOAD;
    typeNames[AlienCommand::USER_LIST] = ICONS::USERS;
    typeNames[AlienCommand::DEFAULT_USER] = ICONS::USERS;
    typeNames[AlienCommand::RESUBMIT] = ICONS::RESUBMIT;
    typeNames[AlienCommand::KILL] = ICONS::KILL;
    typeNames[AlienCommand::RESUBMIT_MASTER] = ICONS::RESUBMIT;
    typeNames[AlienCommand::KILL_MASTER] = ICONS::KILL;
    typeNames[AlienCommand::TRACE] = ICONS::INFORMATION;
    typeNames[AlienCommand::TRACE_ALL] = ICONS::INFORMATION;
    typeNames[AlienCommand::TRACE_JDL] = ICONS::INFORMATION;
    typeNames[AlienCommand::CLEAR_QUEUE] = ICONS::DELETE;

    // load link pixmap
    for( IconNames::iterator iter = typeNames.begin(); iter != typeNames.end(); ++iter )
    { _icons().insert( iter.key(), IconEngine::get( iter.value() ) ); }

    return;
}

//_______________________________________________
AlienCommandModel::IconCache& AlienCommandModel::_icons( void )
{
    static IconCache cache;
    return cache;
}

//____________________________________________________________
const QIcon& AlienCommandModel::_icon( AlienCommand::Type type )
{

    IconCache::iterator iter = ( _icons().find( type ) );
    if( iter != _icons().end() ) return iter.value();
    else return _icons().insert( type, CustomPixmap().empty( IconSize( IconSize::Medium ) ) ).value();

}

//________________________________________________________
bool AlienCommandModel::SortFTor::operator () ( AlienCommand first, AlienCommand second ) const
{

    if( order_ == Qt::DescendingOrder ) std::swap( first, second );
    switch( type_ )
    {
        case AlienCommandModel::PRIORITY:
        if( first.priority() != second.priority() ) return first.priority() < second.priority();
        else break;

        case AlienCommandModel::NAME:
        if( first.type() != second.type() ) return first.type() < second.type();
        else break;

        case AlienCommandModel::ARGUMENTS:
        {
            const QString firstArguments( first.arguments().join( " " ) );
            const QString secondArguments( second.arguments().join( " " ) );
            if( firstArguments != secondArguments ) return firstArguments < secondArguments;
            else break;
        }

        default: break;
    }

    return first.id() < second.id();

}
