#ifndef AlienInterface_h
#define AlienInterface_h

// $Id: AlienInterface.h,v 1.20 2012/09/27 13:46:34 hpereira Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "Debug.h"
#include "File.h"
#include "FileRecord.h"
#include "Job.h"
#include "JobSet.h"
#include "AlienCommand.h"
#include "AlienCommandList.h"
#include "MessageBuffer.h"
#include "UserSet.h"

#include <QtCore/QObject>
#include <QtCore/QProcess>
#include <QtCore/QProcessEnvironment>
#include <QtCore/QString>
#include <QtCore/QMap>

#include <cassert>

class CustomProcess;

//! Job managment
class AlienInterface:public QObject, public Counter
{

    //! Qt meta object declaration
    Q_OBJECT

    public:

    //! constructor
    AlienInterface( QObject* parent = 0 );

    //! destructor
    virtual ~AlienInterface( void );

    //! hide killed jobs
    void setHideKilledJobs( bool value )
    { hideKilledJobs_ = value; }

    signals:

    //! default user is available
    void defaultUserAvailable( QString );

    //! user list is available
    void userListAvailable( UserSet );

    //! job list available
    void jobListAvailable( QString, const JobSet& );

    //! master list is available
    void masterListAvailable( QString, const JobSet& );

    //! children list is available
    void childrenListAvailable( const Job&, const JobSet& );

    //! job status
    void jobStatusAvailable( const Job& );

    //! job information is available
    void jobInformationAvailable( const Job&, const QString& = QString() );

    //! job output files
    void jobFilesAvailable( const Job&, const QString&, const FileRecord::List& );

    //! job JDL is available
    void jdlAvailable( const Job&, const QString& );

    //! command added
    void commandAdded( const AlienCommand& );

    //! commands added
    void commandsAdded( const AlienCommand::List& );

    //! command completed
    void commandCompleted( const AlienCommand& );

    //! command started
    void commandStarted( const AlienCommand& );

    //! emitted when busy state is changed
    void busyStateChanged( bool );

    //! emmitted when busy processing commands
    void busy( void );

    //! emmitted when there is no more commands in pile
    void ready( void );

    //! logAvailable
    void logAvailable( const QString&, unsigned int format = 0 );

    //! message
    void messageAvailable( const QString& );

    public slots:

    //! add command
    void addCommand( const AlienCommand& );

    //! add commands
    void addCommands( const AlienCommand::List& );

    //! abort all commands
    void abort( void );

    //! abort selected commands
    void abortCommands( const AlienCommand::List& );

    //! toggle pause
    void togglePause( bool value )
    {
        paused_ = value;
        if( !paused_ ) _runCommand();
    }

    //! update set of active users
    void updateActiveUsers( UserSet users )
    { activeUsers_ = users; }

    //! clear queue
    void clearQueue( const QString&, const Job::IdSet& );

    protected slots:

    //! run next command in pile
    bool _runCommand( void );

    //! take action when current command completed
    void _commandCompleted( int, QProcess::ExitStatus );

    //! error from process
    void _error( QProcess::ProcessError );

    //! read on-fly from process
    void _readStdout( void );

    //! read on-fly from process
    void _readStderr( void );

    protected:

    //! true if paused
    bool _paused( void ) const
    { return paused_; }

    //! process default user
    void _processDefaultUser( void );

    //! process user list
    void _processUserList( void );

    //! process job list
    void _processJobList( const QString& );

    //! process master job list
    void _processMasterList( const QString& );

    //! process children job list
    void _processChildrenList( const Job& );

    //! process job status
    void _processJobStatus( const Job& );

    //! job output files
    void _processJobFiles( const Job&, const QString& );

    //! process job set
    void _processJobInformation( Job, bool );

    //! process job set
    void _processJdl( const Job& );

    //! reset buffers
    void _resetBuffers( void );

    //! reset process
    void _resetProcess( void );

    //! create new process to run command
    CustomProcess& _newProcess( void );

    //! process
    CustomProcess& _process( void ) const
    {
        assert( process_ );
        return *process_;
    }

    //! attempt at identifying error lines
    bool _isError( const QString& ) const;

    //! find killed job in stored map
    /*! returns invalid job if not found */
    Job _findKilledJob( const QString&, int ) const;

    private slots:

    //! update configuration
    void _updateConfiguration( void );


    private:

    //! true if paused
    bool paused_;

    //! hide killed jobs
    bool hideKilledJobs_;

    //! custom process
    CustomProcess* process_;

    //! stdout buffer
    MessageBuffer stdoutBuffer_;

    //! stderr buffer
    MessageBuffer stderrBuffer_;

    //! map command type to name
    AlienCommandList commands_;

    //! set of active users
    UserSet activeUsers_;

    //! map users to (killed) job set
    typedef QMap< QString, JobSet > JobMap;
    JobMap killedJobs_;

    //! system environment
    QProcessEnvironment systemEnvironment_;

};

#endif
