// $Id: JobInformationDialog.cpp,v 1.42 2012/10/10 13:37:31 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "JobInformationDialog.h"

#include "AnimatedTabWidget.h"
#include "ContextMenu.h"
#include "CustomComboBox.h"
#include "FileRecordProperties.h"
#include "FileDialog.h"
#include "GridLayout.h"
#include "IconEngine.h"
#include "Icons.h"
#include "InformationDialog.h"
#include "JdlSyntaxHighlighter.h"
#include "JobTraceSyntaxHighlighter.h"
#include "OpenFileDialog.h"
#include "TextEditor.h"
#include "TreeView.h"
#include "Util.h"
#include "XmlOptions.h"

#include <QtGui/QShortcut>
#include <QtCore/QDir>

//______________________________________________
JobInformationDialog::JobInformationDialog( QWidget* parent ):
    CustomDialog( parent, CloseButton )
{

    Debug::Throw( "JobInformationDialog::JobInformationDialog.\n" );
    setWindowTitle( "Job Information - Gridwatch" );

    setAttribute( Qt::WA_DeleteOnClose );
    setOptionName( "JOB_INFORMATION_DIALOG" );

    // customize layout
    layout()->setMargin(0);
    layout()->setSpacing(0);
    buttonLayout().setMargin(5);

    // tabwidget
    tabWidget_ = new AnimatedTabWidget( this );
    mainLayout().addWidget( tabWidget_ );

    // general information
    QWidget *box;
    tabWidget_->addTab( box = new QWidget(), "General" );

    QHBoxLayout* hLayout = new QHBoxLayout();
    hLayout->setMargin(5);
    hLayout->setSpacing(5);
    box->setLayout( hLayout );

    // try load Question icon
    QLabel* label = new QLabel(box);
    label->setPixmap( IconEngine::get( ICONS::INFORMATION ).pixmap( iconSize() ) );
    hLayout->addWidget( label, 0, Qt::AlignTop );

    QVBoxLayout* layout = new QVBoxLayout();
    layout->setMargin(0);
    layout->setSpacing( 5 );
    hLayout->addLayout( layout, 1 );

    GridLayout* gridLayout = new GridLayout();
    gridLayout->setSpacing( 5 );
    gridLayout->setMaxCount( 2 );
    gridLayout->setColumnAlignment( 0, Qt::AlignRight|Qt::AlignVCenter );
    layout->addLayout( gridLayout );

    idLabel_ = new Label( "Id:", box, gridLayout );
    parentLabel_ = new Label( "Parent:", box, gridLayout );
    userLabel_ = new Label( "User:", box, gridLayout );
    statusLabel_ = new Label( "Status:", box, gridLayout );
    locationLabel_ = new Label( "Location:", box, gridLayout );
    hostLabel_ = new Label( "Host:", box, gridLayout );
    commandLabel_ = new Label( "Command:", box, gridLayout );
    submissionTimeLabel_ = new Label( "Submitted:", box, gridLayout );
    startTimeLabel_ = new Label( "Started:", box, gridLayout );
    completionTimeLabel_ = new Label( "Completed:", box, gridLayout );

    gridLayout->setColumnStretch( 1, 1 );
    layout->addStretch( 1 );

    // trace
    tabWidget_->addTab( traceEditor_ = new TextEditor(), "Trace" );
    traceEditor_->setReadOnly( true );
    new JobTraceSyntaxHighlighter( traceEditor_->document() );

    // jdl
    tabWidget_->addTab( jdlEditor_ = new TextEditor(), "JDL" );
    jdlEditor_->setReadOnly( true );
    new JdlSyntaxHighlighter( jdlEditor_->document() );

    // output files
    tabWidget_->addTab( filesContainer_ = new QWidget(), "Output Files" );
    layout = new QVBoxLayout();
    layout->setMargin(5);
    layout->setSpacing(5);
    filesContainer_->setLayout( layout );

    hLayout = new QHBoxLayout();
    hLayout->setMargin(0);
    hLayout->setSpacing(5);
    layout->addLayout( hLayout );

    // recycle path
    pathLabel_ = new Label( "Path:", filesContainer_, hLayout );

    // files
    layout->addWidget( list_ = new TreeView( filesContainer_ ) );
    list_->setModel( &model_ );
    list_->setSelectionMode( TreeView::SingleSelection );
    list_->setOptionName( "OUTPUT_FILES_LIST" );
    connect( list_, SIGNAL( activated( QModelIndex ) ), this, SLOT( _openOrSave( void ) ) );

    // reload button
    QPushButton *button = new QPushButton( IconEngine::get( ICONS::RELOAD ), "Reload", this );
    connect( button, SIGNAL( clicked() ), SLOT( requestUpdate() ) );
    buttonLayout().insertWidget(1, button );

    // accelerators
    new QShortcut( QKeySequence::Close, this, SLOT( close() ) );
    new QShortcut( QKeySequence::Refresh, this, SLOT( requestUpdate() ) );

    //! actions
    _installActions();

    QMenu* menu( new ContextMenu( list_ ) );
    menu->addAction( openAction_ );
    menu->addAction( saveAsAction_ );
    connect( list_->selectionModel(), SIGNAL( selectionChanged ( const QItemSelection &, const QItemSelection & ) ), SLOT( _updateActions() ) );

    // enable/disable output files tab
    tabWidget_->setTabEnabled(
        tabWidget_->indexOf( filesContainer_ ),
        false );

}

//_________________________________________________________________________
void JobInformationDialog::setJob( const Job& job )
{

    Debug::Throw( "JobInformationDialog::setJob.\n" );
    assert( job.isValid() );

    // change window title
    QString buffer;
    QTextStream( &buffer ) << "Job " << job.id() << " - gridwatch";
    setWindowTitle( buffer );

    // assign
    job_ = job;

    // fill labels
    idLabel_->setText( QString().setNum(job_.id()) );
    parentLabel_->setText( QString().setNum(job_.parentId()) );
    userLabel_->setText( job_.get( Job::USER ) );
    {
        QString buffer;
        QTextStream( &buffer ) << job_.status() << " (" << job_.get( Job::STATUS ) << ")";
        statusLabel_->setText( buffer );
    }

    locationLabel_->setText( job_.get( Job::LOCATION ) );
    hostLabel_->setText( job_.get( Job::HOST ) );
    commandLabel_->setText( job_.get( Job::COMMAND ) );

    // assign time
    submissionTimeLabel_->setText( job.get( Job::SUB_TIME ) );
    startTimeLabel_->setText( job.get( Job::RUN_TIME ) );
    completionTimeLabel_->setText( job.get( Job::END_TIME ) );

    // clear editors
    traceEditor_->QTextEdit::clear();
    jdlEditor_->QTextEdit::clear();

    // set labels visibility
    const bool isMasterJob( job_.isMasterJob() && job_.hasChildren() );
    parentLabel_->setVisible( !job_.isMasterJob() );
    locationLabel_->setVisible( job.has( Job::LOCATION ) && !isMasterJob );
    hostLabel_->setVisible( job.has( Job::HOST ) && !isMasterJob );

    submissionTimeLabel_->setVisible( job.has( Job::SUB_TIME ) );
    startTimeLabel_->setVisible( job.has( Job::RUN_TIME ) );
    completionTimeLabel_->setVisible( job.has( Job::END_TIME ) );

    // clear output directory
    outputDirectory_.clear();

    // enable/disable output files tab
    tabWidget_->setTabEnabled( tabWidget_->indexOf( filesContainer_ ), job_.isCompleted() );

}

//_________________________________________________________________________
void JobInformationDialog::requestUpdate( void )
{

    Debug::Throw( "JobInformationDialog::requestUpdate.\n" );
    if( !job_.isValid() ) return;

    // request status
    _requestStatus();

    // request children
    //! this is not needed in InformationDialog per-se, but might help fix the Main list
    _requestChildren();

    // request trace
    _requestTrace();

    // request jdl
    _requestJdl();

    // change cursor
    setCursor( Qt::WaitCursor );
    traceEditor_->viewport()->setCursor( Qt::WaitCursor );
    jdlEditor_->viewport()->setCursor( Qt::WaitCursor );


}

//_________________________________________________________________________
void JobInformationDialog::update( const Job& job, const QString& buffer )
{

    Debug::Throw( "JobInformationDialog::update.\n" );

    // check matching
    if( job_.isValid() && job != job_ ) return;

    // reassign parent
    if( job.parentId() >= 0 )
    {
        parentLabel_->setText( QString().setNum( job.parentId() ) );
        parentLabel_->setVisible( !job.isMasterJob() );
    }

    // assign time and trace
    submissionTimeLabel_->setText( job.get( Job::SUB_TIME ) );
    startTimeLabel_->setText( job.get( Job::RUN_TIME ) );
    completionTimeLabel_->setText( job.get( Job::END_TIME ) );

    // set label visibility
    submissionTimeLabel_->setVisible( job.has( Job::SUB_TIME ) );
    startTimeLabel_->setVisible( job.has( Job::RUN_TIME ) );
    completionTimeLabel_->setVisible( job.has( Job::END_TIME ) );

    // log
    if( !buffer.isEmpty() )
    { traceEditor_->setText( buffer ); }

}


//_________________________________________________________________________
void JobInformationDialog::updateStatus( const Job& job )
{

    Debug::Throw( "JobInformationDialog::updateStatus.\n" );

    // check matching
    if( job_.isValid() && job != job_ ) return;

    // copy relevant information to stored job
    job_.setParentId( job.parentId() );
    job_.assign( Job::USER, job.get( Job::USER ) );
    job_.assign( Job::STATUS, job.get( Job::STATUS ) );
    job_.assign( Job::LOCATION, job.get( Job::LOCATION ) );
    job_.assign( Job::HOST, job.get( Job::HOST ) );
    job_.assign( Job::COMMAND, job.get( Job::COMMAND ) );

    // update labels
    idLabel_->setText( QString().setNum(job_.id()) );
    parentLabel_->setText( QString().setNum(job_.parentId()) );
    userLabel_->setText( job_.get( Job::USER ) );
    {
        QString buffer;
        QTextStream( &buffer ) << job_.status() << " (" << job_.get( Job::STATUS ) << ")";
        statusLabel_->setText( buffer );
    }

    locationLabel_->setText( job_.get( Job::LOCATION ) );
    hostLabel_->setText( job_.get( Job::HOST ) );
    commandLabel_->setText( job.get( Job::COMMAND ) );

    // enable/disable output files tab
    tabWidget_->setTabEnabled( tabWidget_->indexOf( filesContainer_ ), job_.isCompleted() );

}

//_________________________________________________________________________
void JobInformationDialog::updateJdl( const Job& job, const QString& buffer )
{
    Debug::Throw( "JobInformationDialog::updateJdl.\n" );

    // check matching
    if( job_.isValid() && job != job_ ) return;

    // check buffer
    if( buffer.isEmpty() ) return;

    // assign to editor
    jdlEditor_->setText( buffer );

    // try load output directory
    QRegExp outputDirRegExp( "OutputDir = \"(\\S+)\";" );
    if( outputDirRegExp.indexIn( buffer ) >= 0 )
    { outputDirectory_ = outputDirRegExp.cap(1); }

    if( job_.isCompleted() && !outputDirectory_.isEmpty() )
    { _requestFiles( outputDirectory_ ); }

}

//_________________________________________________________________________
void JobInformationDialog::updateFiles( const Job& job, const QString& path, const FileRecord::List& files )
{
    Debug::Throw( "JobInformationDialog::updateFiles.\n" );

    // check matching
    if( job_.isValid() && job != job_ ) return;

    Model::List records;
    foreach( const FileRecord& record, files )
    { records << record; }

    pathLabel_->setText( path );
    model_.set( records );
    list_->resizeColumns();

}

//_________________________________________________________________________
void JobInformationDialog::commandCompleted( const AlienCommand& command )
{
    Debug::Throw( "JobInformationDialog::commandCompleted.\n" );
    if( command.type() == AlienCommand::TRACE_JDL && command.jobId() == job_.id() )
    {
        unsetCursor();
        traceEditor_->viewport()->setCursor( Qt::IBeamCursor );
        jdlEditor_->viewport()->setCursor( Qt::IBeamCursor );
    }
}

//_________________________________________________________________________
void JobInformationDialog::_openOrSave( unsigned int mode )
{
    Debug::Throw( "JobInformationDialog::_openOrSave.\n" );
    QModelIndex index( list_->selectionModel()->currentIndex() );
    if( !index.isValid() ) return;

    OpenFileDialog dialog( this, mode );
    dialog.setWindowTitle( "Open Remote File - gridwatch" );

    // set command from Options
    if( mode == OpenFileDialog::Any )
    { dialog.setCommand( (OpenFileDialog::Command) XmlOptions::get().get<int>( "OPEN_COMMAND" ) ); }

    // retrieve applications from options
    foreach( const Option& option, XmlOptions::get().specialOptions( "APPLICATIONS" ) )
    { dialog.actionComboBox().addItem( option.raw() ); }

    if( dialog.centerOnParent().exec() == QDialog::Rejected ) return;

    // save command
    if( mode == OpenFileDialog::Any )
    { XmlOptions::get().set<int>( "OPEN_COMMAND", dialog.command() ); }

    // get remote filename
    File remote( model_.get(index).file() );

    // take action depending on dialog
    if( dialog.command() == OpenFileDialog::SaveAs ) _saveAs( remote );
    else {

        // retrieve application from combobox and add as options
        QString command( dialog.actionComboBox().currentText() );
        if( command.isEmpty() )
        {
            InformationDialog( this, "No command specified to open the selected files. <Open> canceled." ).exec();
            return;
        }

        // update options
        XmlOptions::get().add( "APPLICATIONS", Option( command, Option::Recordable|Option::Current ) );


        // temporary path
        emit scratchFileCreated( Util::tmp() + "/gridwatch" );
        emit scratchFileCreated( Util::tmp() + "/gridwatch/" + Util::user() );

        // generate temporary filename
        QString path;
        QTextStream( &path )
            << Util::tmp() << "/gridwatch/"
            << Util::user()
            << "/alien-job-"
            << job_.id();

        File local( remote.localName().addPath( path ) );
        Debug::Throw() << "JobInformationDialog::_openOrSave - remote: " << remote << endl;
        Debug::Throw() << "JobInformationDialog::_openOrSave - local: " << local << endl;

        // make sure local path exists
        {
            QDir path( local.path() );
            if( !( path.exists() || path.mkpath( "." ) ) ) return;
            emit scratchFileCreated( local.path() );
        }

        // register local file to scratch manager
        emit scratchFileCreated( local );

        // run command
        emit commandRequested( AlienCommand( AlienCommand::COPY )
            << remote
            << local
            << command );

    }

}

//_________________________________________________________________________
void JobInformationDialog::_open( void )
{
    Debug::Throw( "JobInformationDialog::_open.\n" );
    return _openOrSave( OpenFileDialog::OpenWith );
}


//_________________________________________________________________________
void JobInformationDialog::_saveAs( File remote )
{
    Debug::Throw( "JobInformationDialog::_saveAs.\n" );
    if( remote.isEmpty() )
    {

        QModelIndex index( list_->selectionModel()->currentIndex() );
        if( !index.isValid() ) return;
        remote = model_.get(index).file();
    }

    // create file dialog
    FileDialog dialog( this );
    dialog.setFileMode( QFileDialog::AnyFile );
    dialog.setAcceptMode( QFileDialog::AcceptSave );
    dialog.selectFile( remote.localName() );
    File local( dialog.getFile() );
    if( local.isNull() ) return;
    else local = local.expand();

    // check if file is directory
    if( local.isDirectory() )
    {
        QString buffer;
        QTextStream( &buffer ) << "File \"" << local << "\" is a directory. <Save As> canceled.";
        InformationDialog( this, buffer ).exec();
        return;
    }

    // run command
    emit commandRequested( AlienCommand( AlienCommand::COPY )
        << remote
        << local );

}

//__________________________________________________
void JobInformationDialog::_updateActions( void )
{
    const bool hasSelection( !list_->selectionModel()->selectedIndexes().isEmpty() );
    openAction_->setEnabled( hasSelection );
    saveAsAction_->setEnabled( hasSelection );
}

//________________________________________________________________________
void JobInformationDialog::_installActions( void )
{
    Debug::Throw( "JobInformationDialog::_installActions.\n" );
    addAction( openAction_ = new QAction( IconEngine::get( ICONS::OPEN ), "Open With...", this ) );
    connect( openAction_, SIGNAL( triggered( void ) ), SLOT( _open( void ) ) );
    openAction_->setShortcut( QKeySequence::Open );
    openAction_->setEnabled( false );

    addAction( saveAsAction_ = new QAction( IconEngine::get( ICONS::SAVE_AS ), "Save as...", this ) );
    connect( saveAsAction_, SIGNAL( triggered( void ) ), SLOT( _saveAs( void ) ) );
    saveAsAction_->setShortcut( QKeySequence::SaveAs );
    saveAsAction_->setEnabled( false );
}

//________________________________________________________________________
void JobInformationDialog::_requestStatus( void )
{

    Debug::Throw( "JobInformationDialog::_requestStatus.\n" );

    // request trace
    emit commandRequested( AlienCommand( AlienCommand::JOB_STATUS )
        << job_.get( Job::USER )
        << QString().setNum(job_.parentId() )
        << QString().setNum(job_.id()) );

}

//________________________________________________________________________
void JobInformationDialog::_requestChildren( void )
{

    Debug::Throw( "JobInformationDialog::_requestTrace.\n" );

    // request trace
    if( job_.isMasterJob() )
    {
        AlienCommand command( AlienCommand::CHILDREN_LIST );
        command
            << job_.get( Job::USER )
            << QString().setNum( job_.parentId() )
            << QString().setNum( job_.id() );
        emit commandRequested( command );
    }

}

//________________________________________________________________________
void JobInformationDialog::_requestTrace( void )
{

    Debug::Throw( "JobInformationDialog::_requestTrace.\n" );

    // request trace
    emit commandRequested( AlienCommand( AlienCommand::TRACE_ALL )
        << job_.get( Job::USER )
        << QString().setNum(job_.parentId() )
        << QString().setNum(job_.id()) );

}

//________________________________________________________________________
void JobInformationDialog::_requestFiles( const QString& path )
{
    Debug::Throw( "JobInformationDialog::_requestTrace.\n" );
    emit commandRequested( AlienCommand( AlienCommand::JOB_FILES )
        << job_.get( Job::USER )
        << QString().setNum(job_.parentId() )
        << QString().setNum(job_.id())
        << path );
    return;
}

//________________________________________________________________________
void JobInformationDialog::_requestJdl( void )
{
    Debug::Throw( "JobInformationDialog::_requestJdl.\n" );

    // request jdl
    emit commandRequested( AlienCommand( AlienCommand::TRACE_JDL )
        << job_.get( Job::USER )
        << QString().setNum(job_.parentId() )
        << QString().setNum(job_.id()) );

}

//_________________________________________________________________________
QString JobInformationDialog::_recyclePath( void ) const
{
    // get user and check
    const QString user( job_.get( Job::USER ) );
    assert( !user.isEmpty() );

    QString out;
    QTextStream( &out ) << "/alice/cern.ch/user/" << user[0] << "/" << user << "/recycle/alien-job-" << job_.id() << "/";
    return out;
}

//_________________________________________________________________________
JobInformationDialog::Label::Label( const QString& text, QWidget* parent, GridLayout* layout ):
    QObject( parent )
{
    Debug::Throw( "JobInformationLabel::Label" );
    layout->addWidget( key_ = new QLabel( text, parent ) );
    layout->addWidget( value_ = new QLabel( parent ) );
    value_->setTextInteractionFlags( Qt::TextSelectableByMouse|Qt::TextSelectableByKeyboard );
}

//_________________________________________________________________________
JobInformationDialog::Label::Label( const QString& text, QWidget* parent, QBoxLayout* layout ):
    QObject( parent )
{
    Debug::Throw( "JobInformationLabel::Label" );
    layout->addWidget( key_ = new QLabel( text, parent ), 0 );
    layout->addWidget( value_ = new QLabel( parent ), 1 );
    key_->setAlignment( Qt::AlignVCenter|Qt::AlignRight );
    value_->setTextInteractionFlags( Qt::TextSelectableByMouse|Qt::TextSelectableByKeyboard );
}

//_________________________________________________________________________
const QString JobInformationDialog::Model::columnTitles_[ JobInformationDialog::Model::nColumns ] =
{
    "Permissions",
    "Owner",
    "Group",
    "Size",
    "Date",
    "File Name"
};

//__________________________________________________________________
QVariant JobInformationDialog::Model::data( const QModelIndex& index, int role ) const
{

    // check index, role and column
    if( !index.isValid() ) return QVariant();

    // retrieve associated file info
    const FileRecord& record( get(index) );

    // return text associated to file and column
    if( role == Qt::DisplayRole ) {

        switch( index.column() )
        {
            case PERMISSIONS: return record.property( FileRecordProperties::PERMISSIONS );
            case USER: return record.property( FileRecordProperties::USER );
            case GROUP: return record.property( FileRecordProperties::GROUP );
            case SIZE: return File::sizeString( record.property( FileRecordProperties::SIZE ).toInt() );
            case CREATION: return record.property( FileRecordProperties::CREATION );
            case FILE: return record.file().localName();
            default: return QVariant();
        }
    }

    return QVariant();

}

//__________________________________________________________________
QVariant JobInformationDialog::Model::headerData(int section, Qt::Orientation orientation, int role) const
{

    if(
        orientation == Qt::Horizontal &&
        role == Qt::DisplayRole &&
        section >= 0 &&
        section < nColumns )
    { return columnTitles_[section]; }

    // return empty
    return QVariant();

}


//____________________________________________________________
void JobInformationDialog::Model::_sort( int column, Qt::SortOrder order )
{ std::sort( _get().begin(), _get().end(), SortFTor( (ColumnType) column, order ) ); }

//________________________________________________________
bool JobInformationDialog::Model::SortFTor::operator () ( FileRecord first, FileRecord second ) const
{

    if( order_ == Qt::DescendingOrder ) std::swap( first, second );

    switch( type_ )
    {
        case PERMISSIONS: return first.property( FileRecordProperties::PERMISSIONS ) < first.property( FileRecordProperties::PERMISSIONS );
        case USER: return first.property( FileRecordProperties::USER ) < first.property( FileRecordProperties::USER );
        case GROUP: return first.property( FileRecordProperties::GROUP ) < first.property( FileRecordProperties::GROUP );
        case SIZE: return first.property( FileRecordProperties::SIZE ).toInt() < first.property( FileRecordProperties::SIZE ).toInt();
        case CREATION: return first.property( FileRecordProperties::CREATION ) < first.property( FileRecordProperties::CREATION );
        case FILE: return first.file().localName() < first.file().localName();
        default: return true;
    }

}
