// $Id: JobModel.cpp,v 1.20 2012/10/08 17:52:37 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

/*!
\file    JobModel.h
\brief   Job model. Stores job information for display in lists
\author  Hugo Pereira
\version $Revision: 1.20 $
\date    $Date: 2012/10/08 17:52:37 $
*/

#include "JobModel.h"

#include "Color.h"
#include "CustomPixmap.h"
#include "Singleton.h"
#include "XmlOptions.h"

#include <QtGui/QPainter>

//_______________________________________________________________
JobModel::JobModel( QObject* parent ):
    TreeModel<Job>( parent ),
    Counter( "JobModel" )
{
    Debug::Throw( "JobModel::JobModel.\n" );
    Job::installFields();
    JobStatistics::installFields();
    connect( Singleton::get().application(), SIGNAL( configurationChanged() ), SLOT( _updateConfiguration() ) );
    _updateConfiguration();
}

//__________________________________________________________________
QVariant JobModel::data( const QModelIndex& index, int role ) const
{

    // check index, role and column
    if( !index.isValid() ) return QVariant();

    // return text associated to file and column
    if( role == Qt::DisplayRole )
    {

        // retrieve associated file info
        const Job& job( _find( index.internalId() ).get() );
        if( index.column() == Job::ID )
        {
            const int value( job.id() );
            if( value ) return value;
            else return "Orphans";

        } else if( index.column() == Job::STATUS ) return job.status();
        else if( index.column() < Job::nFields ) return job.get( index.column() );
        else if( index.column() - Job::nFields < JobStatistics::nFields && job.hasChildren() )
        {
            const int value = job.childrenStatistics().get( JobStatistics::FieldType(index.column() - Job::nFields) );
            if( value ) return value;
            else return "";
        }

    } else if( role == Qt::DecorationRole && index.column() == Job::STATUS ) {

        // retrieve associated file info
        const Job& job( _find( index.internalId() ).get() );
        if( !job.id() ) return QVariant();
        else if( job.isCompleted() ) return completedJobIcon_;
        else if( job.isRunning() ) return runningJobIcon_;
        else if( job.isFailed() ) return failedJobIcon_;
        else return pendingJobIcon_;

    }

    return QVariant();

}

//__________________________________________________________________
QVariant JobModel::headerData(int section, Qt::Orientation orientation, int role) const
{

    if( orientation == Qt::Horizontal && role == Qt::DisplayRole &&  section >= 0 )
    {
        if( section < int(Job::nFields) ) return Job::fieldName( section );
        else if( section - Job::nFields < JobStatistics::nFields ) return JobStatistics::fieldName( section - Job::nFields );
    }

    // return empty
    return QVariant();

}

//________________________________________________________
bool JobModel::SortFTor::operator () ( Job first, Job second ) const
{

    if( order_ == Qt::DescendingOrder ) std::swap( first, second );
    if( type_ >= Job::nFields && first.childrenStatistics().get(type_-Job::nFields) != second.childrenStatistics().get(type_-Job::nFields) ) {

        return first.childrenStatistics().get(type_-Job::nFields) < second.childrenStatistics().get(type_-Job::nFields);

    } else if( first.get( type_ ) != second.get( type_ ) ) {

        switch( type_ )
        {
            case Job::ID: return first.id() < second.id();
            case Job::SUB_TIME: return first.subTime() < second.subTime();
            case Job::RUN_TIME: return first.runTime() < second.runTime();
            case Job::END_TIME: return first.endTime() < second.endTime();
            default: return first.get( type_ ) < second.get( type_ );
        }

    } else return first < second;

}

//__________________________________________________________
void JobModel::_updateConfiguration(void)
{

    Debug::Throw( "JobModel::_updateConfiguration.\n" );
    runningJobIcon_ = _createIcon( BASE::Color( XmlOptions::get().get<QString>("RUNNING_JOB_COLOR") ) );
    completedJobIcon_ = _createIcon( BASE::Color( XmlOptions::get().get<QString>("COMPLETED_JOB_COLOR") ) );
    failedJobIcon_ = _createIcon( BASE::Color( XmlOptions::get().get<QString>("FAILED_JOB_COLOR") ) );

    // empty icon for pending jobs
    unsigned int iconSize = qMax<unsigned int>( XmlOptions::get().get<unsigned int>( "LIST_ICON_SIZE" ), 16 );
    pendingJobIcon_ = QIcon( CustomPixmap().empty( QSize( iconSize, iconSize ) ) );

}

//____________________________________________________________
QIcon JobModel::_createIcon( const QColor& color ) const
{

    Debug::Throw( "JobModel::_createIcon" );

    unsigned int iconSize = qMax<unsigned int>( XmlOptions::get().get<unsigned int>( "LIST_ICON_SIZE" ), 16 );
    double pixmapSize = 0.75* XmlOptions::get().get<double>( "LIST_ICON_SIZE" );
    double offset = 0.5*( iconSize - pixmapSize );

    CustomPixmap pixmap( CustomPixmap().empty( QSize( iconSize, iconSize ) ) );

    if( color.isValid() )
    {

        QPainter painter( &pixmap );
        painter.setRenderHints(QPainter::Antialiasing );
        painter.setPen( Qt::NoPen );

        QRectF rect( QPointF( offset, offset ), QSizeF( pixmapSize, pixmapSize ) );
        QLinearGradient gradient(rect.topLeft(), rect.bottomRight());
        gradient.setColorAt(0, color );
        gradient.setColorAt(1, color.light(135) );
        painter.setBrush( gradient );
        painter.drawEllipse( rect );
        painter.end();

    }

    return QIcon( pixmap );

}
