// $Id: SummaryBar.cpp,v 1.7 2012/10/08 17:52:37 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

/*!
\file SummaryBar.cc
\brief display command progress
\author Hugo Pereira
\version $Revision: 1.7 $
\date $Date: 2012/10/08 17:52:37 $
*/

#include "SummaryBar.h"

#include "Color.h"
#include "Debug.h"
#include "Singleton.h"
#include "XmlOptions.h"

#include <QtGui/QStyleOptionProgressBarV2>
#include <QtGui/QStylePainter>

//___________________________________________________________
SummaryBar::SummaryBar( QWidget* parent ):
    QProgressBar( parent ),
    Counter( "SummaryBar" )
{
    Debug::Throw( "SummaryBar::SummaryBar.\n" );
    setSizePolicy( QSizePolicy( QSizePolicy::Minimum, QSizePolicy::Fixed ) );
    connect( Singleton::get().application(), SIGNAL( configurationChanged() ), SLOT( _updateConfiguration() ) );
    setFixedHeight( fontMetrics().lineSpacing() + 6 );
    setTextVisible( false );
    _updateConfiguration();
}

//___________________________________________________________
void SummaryBar::updateJobStatistics( JobStatistics statistics )
{
    Debug::Throw( "SummaryBar::updateJobStatistics" );
    statistics_ = statistics;
    update();
}

//___________________________________________________________
void SummaryBar::paintEvent( QPaintEvent* event )
{
    QStylePainter painter(this);
    painter.setClipRegion( event->region() );
    QStyleOptionProgressBarV2 opt;
    initStyleOption(&opt);
    opt.text = "";
    opt.maximum = qMax<unsigned int>(statistics_.get( JobStatistics::TOTAL ), 1);

    // draw container
    QStyleOptionProgressBarV2 subOpt(opt);
    subOpt.rect = style()->subElementRect( QStyle::SE_ProgressBarGroove, &opt, this);
    style()->drawControl( QStyle::CE_ProgressBarGroove, &subOpt, &painter, this);
    if( !statistics_.get( JobStatistics::TOTAL ) ) return;

    #if defined(Q_WS_MAC)

    // for mac style, the progressbar color cannot be changed (because native macintosh methods are called).
    // as a fall-back solution, the progressbar only represents the number of completed jobs
    subOpt.progress = statistics_.get( JobStatistics::COMPLETED );
    subOpt.rect = style()->subElementRect( QStyle::SE_ProgressBarContents, &opt, this);
    style()->drawControl( QStyle::CE_ProgressBarContents, &subOpt, &painter, this);

    #else

    // draw running jobs
    if( statistics_.get( JobStatistics::RUNNING ) > 0 )
    {
        subOpt.progress = statistics_.get( JobStatistics::RUNNING ) + statistics_.get( JobStatistics::FAILED ) + statistics_.get( JobStatistics::COMPLETED );
        subOpt.palette.setColor( QPalette::Highlight, runningJobColor_ );
        style()->drawControl( QStyle::CE_ProgressBarContents, &subOpt, &painter, this);
    }

    // draw failed jobs
    if( statistics_.get( JobStatistics::FAILED ) > 0 )
    {
        subOpt.progress = statistics_.get( JobStatistics::COMPLETED ) + statistics_.get( JobStatistics::FAILED );
        subOpt.palette.setColor( QPalette::Highlight, failedJobColor_ );
        style()->drawControl( QStyle::CE_ProgressBarContents, &subOpt, &painter, this);
    }

    // draw completed jobs
    if( statistics_.get( JobStatistics::COMPLETED ) > 0 )
    {
        const int value( statistics_.get( JobStatistics::COMPLETED ) );
        subOpt.progress = value;
        subOpt.palette.setColor( QPalette::Highlight, completedJobColor_ );
        style()->drawControl( QStyle::CE_ProgressBarContents, &subOpt, &painter, this);
    }
    #endif

    painter.end();
    return;

}

//___________________________________________________________
void SummaryBar::_updateConfiguration( void )
{
    Debug::Throw( "SummaryBar::_updateConfiguration.\n" );

    // running job color
    runningJobColor_ = BASE::Color( XmlOptions::get().get<QString>("RUNNING_JOB_COLOR") );
    if( !runningJobColor_.isValid() ) runningJobColor_ = palette().color( QPalette::Highlight );

    // completed job color
    completedJobColor_ = BASE::Color( XmlOptions::get().get<QString>("COMPLETED_JOB_COLOR") );
    if( !completedJobColor_.isValid() ) completedJobColor_ = palette().color( QPalette::Highlight );

    // failed job color
    failedJobColor_ = BASE::Color( XmlOptions::get().get<QString>("FAILED_JOB_COLOR") );
    if( !failedJobColor_.isValid() ) failedJobColor_ = palette().color( QPalette::Highlight );

}
